#include "corde.h"
using namespace Imagine;

Corde cree_corde(int nb) {
    Corde c;
    c.y = new float[nb+1];
    c.n = nb;
    return c;
}

void detruis_corde(Corde corde) {
    delete [] corde.y;
}

Corde copie_corde(Corde corde) {
    Corde c = cree_corde(corde.n);
    for(int i=0; i<=c.n; i++) // n+1 points
        c.y[i] = corde.y[i];
    return c;
}

void double_corde(Corde& corde) {
    Corde c = cree_corde(2*corde.n);
    for(int i=0; i<=corde.n; i++) // copie
        c.y[2*i] = corde.y[i];
    for(int i=1; i<=c.n; i+=2) // interpolation
        c.y[i] = (c.y[i-1]+c.y[i+1])/2;
    detruis_corde(corde);
    corde = c;
}

void dessine_corde(Corde corde, Color col) {
    int ox=10, oy=largeur/2;
    float sx=(largeur-ox-ox)/(float)corde.n; // echelle
    fillCircle(ox, oy, 3, col); // pour faire joli...
    for(int i=1; i<=corde.n; i++)
        drawLine(ox+(int)(sx*(i-1)), oy-(int)(corde.y[i-1]*sx*corde.n),
                 ox+(int)(sx*i),     oy-(int)(corde.y[i]  *sx*corde.n), col);
}

void pince_corde(Corde corde, float pos, float v) {
    assert(0<=pos && pos<=1);
    int j=(int)(pos*corde.n);
    for(int i=0; i<j; i++)
        corde.y[i] = v * i/j;
    corde.y[j]=v;
    for(int i=j+1; i<=corde.n; i++)
        corde.y[i] = v * (i-corde.n)/(j-corde.n);
}

Corde operator+(Corde corde1, Corde corde2) {
    assert(corde1.n == corde2.n);
    Corde c = cree_corde(corde1.n);
    for(int i=0; i<=c.n; i++)
        c.y[i] = corde1.y[i]+corde2.y[i];
    return c;
}

// Applique une iteration du schema numerique. 
// Entree: corde(t-1), prec(t-2).
// Sortie: corde(t)  , prec(t-1).
void vibre_corde(Corde& corde, Corde& prec) {
    Corde nouvelle = cree_corde(corde.n);

    for(int i=1; i<corde.n; i++) {
        float n = corde.y[i+1];
        float p = corde.y[i-1];
        nouvelle.y[i] = 2*corde.y[i]-prec.y[i]+0.5*(n+p-2*corde.y[i]);
    }
    nouvelle.y[0] = corde.y[0];
    nouvelle.y[corde.n] = corde.y[corde.n];

    detruis_corde(prec); // ne pas oublier
    prec=corde;
    corde=nouvelle;
}
